"use strict";

require("core-js/modules/es.object.define-property.js");
var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.serializeArgs = exports.serialize = exports.addSerializer = void 0;
require("core-js/modules/es.array.find.js");
require("core-js/modules/es.array.join.js");
require("core-js/modules/es.array.map.js");
require("core-js/modules/es.array.push.js");
require("core-js/modules/es.function.name.js");
require("core-js/modules/es.object.to-string.js");
require("core-js/modules/es.regexp.exec.js");
require("core-js/modules/es.string.match.js");
require("core-js/modules/esnext.async-iterator.map.js");
require("core-js/modules/esnext.iterator.map.js");
var _isPlainObject = _interopRequireDefault(require("lodash/isPlainObject"));
var _misc = require("../../utilities/misc");
var serializers = [];
var fileSerializer = {
  match: function match(obj) {
    return obj instanceof File;
  },
  serialize: function serialize(file) {
    return (0, _misc.jsonstring)({
      filename: file.name,
      size: file.size
    });
  }
};
var plainObjectSerializer = {
  match: function match(obj) {
    return (0, _isPlainObject["default"])(obj);
  },
  serialize: function serialize(obj) {
    return (0, _misc.jsonstring)(obj);
  }
};
var functionSerializer = {
  match: function match(obj) {
    return typeof obj === 'function';
  },
  serialize: function serialize(obj) {
    return '[Function]';
  }
};
var addSerializer = exports.addSerializer = function addSerializer(serializer) {
  serializers.push(serializer);
};
var serialize = exports.serialize = function serialize(data) {
  if (typeof data === 'string') {
    return data;
  }
  var serializer = serializers.find(function (serializer) {
    return serializer.match(data);
  });
  if (serializer) {
    return serializer.serialize(data);
  }
  return "".concat(data);
};
var serializeArgs = exports.serializeArgs = function serializeArgs(args) {
  return "".concat(args.map(serialize).join(' '));
};
addSerializer(fileSerializer);
addSerializer(plainObjectSerializer);
addSerializer(functionSerializer);