"use strict";

require("core-js/modules/es.symbol.js");
require("core-js/modules/es.symbol.description.js");
require("core-js/modules/es.symbol.iterator.js");
require("core-js/modules/es.error.cause.js");
require("core-js/modules/es.error.to-string.js");
require("core-js/modules/es.array.from.js");
require("core-js/modules/es.array.is-array.js");
require("core-js/modules/es.array.slice.js");
require("core-js/modules/es.function.name.js");
require("core-js/modules/es.object.define-property.js");
require("core-js/modules/es.object.to-string.js");
require("core-js/modules/es.regexp.exec.js");
require("core-js/modules/es.regexp.test.js");
require("core-js/modules/es.regexp.to-string.js");
var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.writeLog = exports.setMaxRecords = exports.deleteLogs = exports.collectLogs = void 0;
var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));
var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));
require("core-js/modules/es.array.concat.js");
require("core-js/modules/es.array.iterator.js");
require("core-js/modules/es.array.push.js");
require("core-js/modules/es.array.reverse.js");
require("core-js/modules/es.date.to-string.js");
require("core-js/modules/es.string.iterator.js");
require("core-js/modules/web.dom-collections.iterator.js");
var _debounce = _interopRequireDefault(require("lodash/debounce"));
var _db = require("./db");
var _mutex = require("../mutex");
var _constants = require("../constants");
var _reply = require("./reply");
var _raceCondition = require("../../utilities/race-condition");
var _jszip = _interopRequireDefault(require("jszip"));
function _createForOfIteratorHelper(r, e) { var t = "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (!t) { if (Array.isArray(r) || (t = _unsupportedIterableToArray(r)) || e && r && "number" == typeof r.length) { t && (r = t); var _n = 0, F = function F() {}; return { s: F, n: function n() { return _n >= r.length ? { done: !0 } : { done: !1, value: r[_n++] }; }, e: function e(r) { throw r; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var o, a = !0, u = !1; return { s: function s() { t = t.call(r); }, n: function n() { var r = t.next(); return a = r.done, r; }, e: function e(r) { u = !0, o = r; }, f: function f() { try { a || null == t["return"] || t["return"](); } finally { if (u) throw o; } } }; }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
var MAX_WAIT_TIME = 3000;
var mutex = new _mutex.Mutex();
var maxRecords = 50000;
var logsCount = 0;
var logs = [];
var writeLog = exports.writeLog = function writeLog(self, callId, log) {
  logsCount++;
  log.content = "[".concat(logsCount, "]").concat(log.content);
  logs.push(log);
  debouncedWriting();
  // reply(self, callId, WorkerDirectives.WRITE_LOG, { isSuccess: true });
};
var debouncedWriting = (0, _debounce["default"])(function () {
  mutex.dispatch(function () {
    return (0, _raceCondition.timeout)((0, _asyncToGenerator2["default"])(/*#__PURE__*/_regenerator["default"].mark(function _callee() {
      var time1, logscopy, count, exceedNo, total, pk, time2;
      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            time1 = new Date().getTime();
            logscopy = logs;
            logs = [];
            _context.prev = 3;
            _context.next = 6;
            return _db.db.logs.count();
          case 6:
            count = _context.sent;
            exceedNo = logscopy.length + count - maxRecords;
            if (!(exceedNo > 0)) {
              _context.next = 15;
              break;
            }
            console.log("[worker] the count of stored logs exceeded limit of ".concat(maxRecords, " for ").concat(exceedNo, "."));
            console.log("[worker] start deleting records. exceeded: ".concat(exceedNo));
            _context.next = 13;
            return _db.db.logs.orderBy(':id').reverse().limit(exceedNo)["delete"]();
          case 13:
            total = _context.sent;
            console.log("[worker] done deleting records. total: ".concat(total));
          case 15:
            _context.next = 20;
            break;
          case 17:
            _context.prev = 17;
            _context.t0 = _context["catch"](3);
            console.error("[worker] clear db failed.", _context.t0);
          case 20:
            _context.prev = 20;
            console.log("[worker] start writing logs. total: ".concat(logscopy.length));
            _context.next = 24;
            return _db.db.logs.bulkPut(logscopy);
          case 24:
            pk = _context.sent;
            time2 = new Date().getTime();
            console.log("[worker] done writing logs in ".concat(time2 - time1, "ms. total: ").concat(logscopy.length, ", lastId: ").concat(pk));
            _context.next = 32;
            break;
          case 29:
            _context.prev = 29;
            _context.t1 = _context["catch"](20);
            console.error("[worker] bulkPut ".concat(logscopy.length, " failed."), _context.t1);
          case 32:
          case "end":
            return _context.stop();
        }
      }, _callee, null, [[3, 17], [20, 29]]);
    }))(), MAX_WAIT_TIME);
  })["catch"](function (e) {
    console.log('[worker] an error during dispatching writeLog', e);
  });
}, 2 * 1000, {
  maxWait: 30 * 1000
});
var collectLogs = exports.collectLogs = /*#__PURE__*/function () {
  var _ref2 = (0, _asyncToGenerator2["default"])(/*#__PURE__*/_regenerator["default"].mark(function _callee2(self, callId, labels) {
    var blob, lastId, _iterator, _step, _loop, buffer;
    return _regenerator["default"].wrap(function _callee2$(_context3) {
      while (1) switch (_context3.prev = _context3.next) {
        case 0:
          blob = new Blob();
          lastId = 0;
          _context3.prev = 2;
          _iterator = _createForOfIteratorHelper(labels);
          _context3.prev = 4;
          _loop = /*#__PURE__*/_regenerator["default"].mark(function _loop() {
            var label, logsStr, count, zip;
            return _regenerator["default"].wrap(function _loop$(_context2) {
              while (1) switch (_context2.prev = _context2.next) {
                case 0:
                  label = _step.value;
                  logsStr = '';
                  count = 0;
                  _context2.next = 5;
                  return _db.db.logs.where('label').equals(label).each(function (e) {
                    count++;
                    logsStr += e.content + '\n';
                    if (e.id > lastId) lastId = e.id;
                  });
                case 5:
                  zip = new _jszip["default"]();
                  zip.file("".concat(label, "-logs.txt"), logsStr);
                  _context2.next = 9;
                  return zip.generateAsync({
                    type: 'blob'
                  });
                case 9:
                  blob = _context2.sent;
                  console.log("[worker] collected ".concat(count, " logs by labels [").concat(label, "]."));
                case 11:
                case "end":
                  return _context2.stop();
              }
            }, _loop);
          });
          _iterator.s();
        case 7:
          if ((_step = _iterator.n()).done) {
            _context3.next = 11;
            break;
          }
          return _context3.delegateYield(_loop(), "t0", 9);
        case 9:
          _context3.next = 7;
          break;
        case 11:
          _context3.next = 16;
          break;
        case 13:
          _context3.prev = 13;
          _context3.t1 = _context3["catch"](4);
          _iterator.e(_context3.t1);
        case 16:
          _context3.prev = 16;
          _iterator.f();
          return _context3.finish(16);
        case 19:
          _context3.next = 25;
          break;
        case 21:
          _context3.prev = 21;
          _context3.t2 = _context3["catch"](2);
          console.error("[worker] failed to collect logs.", _context3.t2);
          (0, _reply.reply)(self, callId, _constants.WorkerDirectives.COLLECT_LOGS, {
            isSuccess: false
          });
        case 25:
          console.log("[worker] completed collecting logs, compressed size: ".concat(blob.size, " bytes."));
          _context3.next = 28;
          return blob.arrayBuffer();
        case 28:
          buffer = _context3.sent;
          (0, _reply.reply)(self, callId, _constants.WorkerDirectives.COLLECT_LOGS, {
            isSuccess: true,
            buffer: buffer,
            lastId: lastId
          }, [buffer]);
        case 30:
        case "end":
          return _context3.stop();
      }
    }, _callee2, null, [[2, 21], [4, 13, 16, 19]]);
  }));
  return function collectLogs(_x, _x2, _x3) {
    return _ref2.apply(this, arguments);
  };
}();
var deleteLogs = exports.deleteLogs = /*#__PURE__*/function () {
  var _ref3 = (0, _asyncToGenerator2["default"])(/*#__PURE__*/_regenerator["default"].mark(function _callee3(self, callId, labels, lastId) {
    var count;
    return _regenerator["default"].wrap(function _callee3$(_context4) {
      while (1) switch (_context4.prev = _context4.next) {
        case 0:
          _context4.prev = 0;
          _context4.next = 3;
          return _db.db.logs.where('id').belowOrEqual(lastId)["delete"]();
        case 3:
          count = _context4.sent;
          console.log("[logger] deleted ".concat(count, " logs from ").concat(lastId, "."));
          (0, _reply.reply)(self, callId, _constants.WorkerDirectives.DELETE_LOGS, {
            isSuccess: true
          });
          _context4.next = 12;
          break;
        case 8:
          _context4.prev = 8;
          _context4.t0 = _context4["catch"](0);
          console.error("[logger] failed to delete logs.", _context4.t0);
          (0, _reply.reply)(self, callId, _constants.WorkerDirectives.DELETE_LOGS, {
            isSuccess: false
          });
        case 12:
        case "end":
          return _context4.stop();
      }
    }, _callee3, null, [[0, 8]]);
  }));
  return function deleteLogs(_x4, _x5, _x6, _x7) {
    return _ref3.apply(this, arguments);
  };
}();
var setMaxRecords = exports.setMaxRecords = function setMaxRecords(self, callId, max) {
  maxRecords = max;
  (0, _reply.reply)(self, callId, _constants.WorkerDirectives.SET_MAX_RECORDS, {
    isSuccess: true
  });
};