"use strict";

require("core-js/modules/es.object.define-property.js");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.installHandler = void 0;
require("core-js/modules/es.date.now.js");
require("core-js/modules/es.date.to-string.js");
var _log = require("./handler/log");
var _binary = require("./handler/binary");
var _fsLog = require("./handler/fs-log");
var _constants = require("./constants");
var _reply = require("./handler/reply");
var installHandler = exports.installHandler = function installHandler(self) {
  console.log("[worker] worker initialzied.");
  self.addEventListener("message", function (evt) {
    if (evt.data) {
      var _evt$data = evt.data,
        type = _evt$data.type,
        callId = _evt$data.callId;
      if (type === _constants.WorkerDirectives.WRITE_LOG) {
        var now = new Date();
        var _evt$data$data = evt.data.data,
          label = _evt$data$data.label,
          message = _evt$data$data.message;
        (0, _log.writeLog)(self, callId, {
          label: label,
          content: message,
          created_at: +Date.now(),
          timestamp: "".concat(now)
        });
      } else if (type === _constants.WorkerDirectives.SET_MAX_RECORDS) {
        (0, _log.setMaxRecords)(self, callId, evt.data.data.maxRecords || 50000);
      } else if (type === _constants.WorkerDirectives.COLLECT_LOGS) {
        var _evt$data$data$labels = evt.data.data.labels,
          labels = _evt$data$data$labels === void 0 ? [] : _evt$data$data$labels;
        (0, _log.collectLogs)(self, callId, labels);
      } else if (type === _constants.WorkerDirectives.DELETE_LOGS) {
        var _evt$data$data2 = evt.data.data,
          _evt$data$data2$label = _evt$data$data2.labels,
          _labels = _evt$data$data2$label === void 0 ? [] : _evt$data$data2$label,
          lastId = _evt$data$data2.lastId;
        (0, _log.deleteLogs)(self, callId, _labels, lastId);
      } else if (type === _constants.WorkerDirectives.PERSIST_BUFFER) {
        var _evt$data$data3 = evt.data.data,
          id = _evt$data$data3.id,
          buffer = _evt$data$data3.buffer;
        (0, _binary.persistBinary)(self, callId, id, new Blob([buffer]));
      } else if (type === _constants.WorkerDirectives.UNPERSIST_BUFFER) {
        var _id = evt.data.data.id;
        (0, _binary.unpersistBinary)(self, callId, _id);
      } else if (type === _constants.WorkerDirectives.GET_BUFFER) {
        var _id2 = evt.data.data.id;
        (0, _binary.getBuffer)(self, callId, _id2);
      } else if (type === _constants.WorkerDirectives.CLEAR_TEMP_LOGS) {
        (0, _binary.clearTempLogs)(self, callId);
      } else if (type === _constants.WorkerDirectives.COLLECT_LOGS_ON_FS) {
        var logDirPath = evt.data.data.logDirPath;
        (0, _fsLog.collectLogsOnFs)(self, callId, logDirPath);
      } else {
        unsupported(self, callId, type);
      }
    }
  });
};
var unsupported = function unsupported(self, callId, directive) {
  (0, _reply.reply)(self, callId, directive, {
    isSuccess: false,
    message: "unsupported directive: ".concat(directive)
  });
};